@@ -5,28 +5,24 @@ The Stata input tools were originally written by Joe Presbrey as part of PyDTA.
 
 You can find more information here http://presbrey.mit.edu/PyDTA
 
-See also
----------
+See Also
+--------
 numpy.lib.io
 """
+import warnings
+
 from statsmodels.compat.python import (zip, lzip, lmap, lrange, string_types, long, lfilter,
-                                asbytes, asstr, range)
+                                       asbytes, asstr, range, PY3)
 from struct import unpack, calcsize, pack
 from struct import error as struct_error
 import datetime
 import sys
+
 import numpy as np
-from numpy.lib._iotools import _is_string_like, easy_dtype
 import statsmodels.tools.data as data_util
 from pandas import isnull
-
-
-def is_py3():
-    import sys
-    if sys.version_info[0] == 3:
-        return True
-    return False
-PY3 = is_py3()
+from pandas.io.stata import StataMissingValue
+from statsmodels.iolib.openfile import get_file_obj
 
 _date_formats = ["%tc", "%tC", "%td", "%tw", "%tm", "%tq", "%th", "%ty"]
 
@@ -149,41 +145,9 @@ def _stata_elapsed_date_to_datetime(date, fmt):
     else:
         raise ValueError("Date fmt %s not understood" % fmt)
 
+
 ### Helper classes for StataReader ###
 
-class _StataMissingValue(object):
-    """
-    An observation's missing value.
-
-    Parameters
-    -----------
-    offset
-    value
-
-    Attributes
-    ----------
-    string
-    value
-
-    Notes
-    -----
-    More information: <http://www.stata.com/help.cgi?missing>
-    """
-
-    def __init__(self, offset, value):
-        self._value = value
-        if isinstance(value, (int, long)):
-            self._str = value-offset is 1 and \
-                    '.' or ('.' + chr(value-offset+96))
-        else:
-            self._str = '.'
-    string = property(lambda self: self._str, doc="The Stata representation of \
-the missing value: '.', '.a'..'.z'")
-    value = property(lambda self: self._value, doc='The binary representation \
-of the missing value.')
-    def __str__(self): return self._str
-    __str__.__doc__ = string.__doc__
-
 class _StataVariable(object):
     """
     A dataset variable.  Not intended for public use.
@@ -193,7 +157,7 @@ class _StataVariable(object):
     variable_data
 
     Attributes
-    -----------
+    ----------
     format : str
         Stata variable format.  See notes for more information.
     index : int
@@ -251,9 +215,11 @@ class StataReader(object):
         Used for Python 3 only. Encoding to use when reading the .dta file.
         Defaults to `locale.getpreferredencoding`
 
-    See also
+    See Also
     --------
-    statsmodels.lib.io.genfromdta
+    statsmodels.iolib.foreign.genfromdta
+    pandas.read_stata
+    pandas.io.stata.StataReader
 
     Notes
     -----
@@ -299,7 +265,13 @@ class StataReader(object):
             (-1.798e+308, +8.988e+307) }
 
     def __init__(self, fname, missing_values=False, encoding=None):
-        if encoding == None:
+        warnings.warn(
+            "StataReader is deprecated as of 0.10.0 and will be removed in a "
+            "future version.  Use pandas.read_stata or "
+            "pandas.io.stata.StataReader instead.",
+            FutureWarning)
+
+        if encoding is None:
             import locale
             self._encoding = locale.getpreferredencoding()
         else:
@@ -382,7 +354,7 @@ class StataReader(object):
         Notes
         -----
         If missing_values is True during instantiation of StataReader then
-        observations with _StataMissingValue(s) are not filtered and should
+        observations with StataMissingValue(s) are not filtered and should
         be handled by your applcation.
         """
 
@@ -513,7 +485,7 @@ class StataReader(object):
         if len(self._col_sizes) == 0:
             self._col_sizes = lmap(lambda x: self._calcsize(x),
                     self._header['typlist'])
-        if k == None:
+        if k is None:
             return self._col_sizes
         else:
             return self._col_sizes[k]
@@ -524,7 +496,7 @@ class StataReader(object):
             nmin, nmax = self.MISSING_VALUES[fmt[-1]]
             if d < nmin or d > nmax:
                 if self._missing_values:
-                    return _StataMissingValue(nmax, d)
+                    return StataMissingValue(nmax, d)
                 else:
                     return None
         return d
@@ -546,15 +518,6 @@ class StataReader(object):
                 self._file.read(self._col_size(i))),
                 lrange(self._header['nvar']))
 
-def _open_file_binary_write(fname, encoding):
-    if hasattr(fname, 'write'):
-        #if 'b' not in fname.mode:
-        return fname
-    if PY3:
-        return open(fname, "wb", encoding=encoding)
-    else:
-        return open(fname, "wb")
-
 def _set_endianness(endianness):
     if endianness.lower() in ["<", "little"]:
         return "<"
@@ -581,8 +544,9 @@ def _dtype_to_stata_type(dtype):
     #TODO: expand to handle datetime to integer conversion
     if dtype.type == np.string_:
         return chr(dtype.itemsize)
-    elif dtype.type == np.object_: # try to coerce it to the biggest string
-                        # not memory efficient, what else could we do?
+    elif dtype.type == np.object_:
+        # try to coerce it to the biggest string
+        # not memory efficient, what else could we do?
         return chr(244)
     elif dtype == np.float64:
         return chr(255)
@@ -731,6 +695,11 @@ class StataWriter(object):
                        'd': 8.98846567431158e+307}
     def __init__(self, fname, data, convert_dates=None, encoding="latin-1",
                  byteorder=None):
+        warnings.warn(
+            "StataWriter is deprecated as of 0.10.0 and will be removed in a "
+            "future version.  Use pandas.DataFrame.to_stata or "
+            "pandas.io.stata.StatWriter instead.",
+            FutureWarning)
 
         self._convert_dates = convert_dates
         # attach nobs, nvars, data, varlist, typlist
@@ -755,7 +724,7 @@ class StataWriter(object):
             byteorder = sys.byteorder
         self._byteorder = _set_endianness(byteorder)
         self._encoding = encoding
-        self._file = _open_file_binary_write(fname, encoding)
+        self._file = get_file_obj(fname, 'wb', encoding)
 
     def _write(self, to_write):
         """
@@ -771,7 +740,7 @@ class StataWriter(object):
         dtype = data.dtype
         descr = dtype.descr
         if dtype.names is None:
-            varlist = _default_names(nvar)
+            varlist = _default_names(self.nvar)
         else:
             varlist = dtype.names
 
@@ -934,6 +903,8 @@ class StataWriter(object):
                     self._write(var)
                 else:
                     try:
+                        if typ in _type_converters:
+                            var = _type_converters[typ](var)
                         self._write(pack(byteorder+TYPE_MAP[typ], var))
                     except struct_error:
                         # have to be strict about type pack won't do any
@@ -999,12 +970,17 @@ def genfromdta(fname, missing_flt=-999., encoding=None, pandas=False,
         If convert_dates is True, then Stata formatted dates will be converted
         to datetime types according to the variable's format.
     """
+    warnings.warn(
+        "genfromdta is deprecated as of 0.10.0 and will be removed in a "
+        "future version.  Use pandas.read_stata instead.",
+        FutureWarning)
+
     if isinstance(fname, string_types):
         fhd = StataReader(open(fname, 'rb'), missing_values=False,
-                encoding=encoding)
+                          encoding=encoding)
     elif not hasattr(fname, 'read'):
         raise TypeError("The input should be a string or a filehandle. "\
-                "(got %s instead)" % type(fname))
+                        "(got %s instead)" % type(fname))
     else:
         fhd = StataReader(fname, missing_values=False, encoding=encoding)
 #    validate_names = np.lib._iotools.NameValidator(excludelist=excludelist,
@@ -1019,8 +995,8 @@ def genfromdta(fname, missing_flt=-999., encoding=None, pandas=False,
     varnames = header['varlist']
     fmtlist = header['fmtlist']
     dataname = header['data_label']
-    labels = header['vlblist'] # labels are thrown away unless DataArray
-                               # type is used
+    labels = header['vlblist']  # labels are thrown away unless DataArray
+                                # type is used
     data = np.zeros((nobs,numvars))
     stata_dta = fhd.dataset()
 
@@ -1054,8 +1030,8 @@ def genfromdta(fname, missing_flt=-999., encoding=None, pandas=False,
         # make the dtype for the datetime types
         cols = np.where(lmap(lambda x : x in _date_formats, fmtlist))[0]
         dtype = data.dtype.descr
-        dtype = [(dt[0], object) if i in cols else dt for i,dt in
-                 enumerate(dtype)]
+        dtype = [(sub_dtype[0], object) if i in cols else sub_dtype
+                  for i, sub_dtype in enumerate(dtype)]
         data = data.astype(dtype) # have to copy
         for col in cols:
             def convert(x):
@@ -1155,58 +1131,43 @@ def savetxt(fname, X, names=None, fmt='%.18e', delimiter=' '):
 
     """
 
-    if _is_string_like(fname):
-        if fname.endswith('.gz'):
-            import gzip
-            fh = gzip.open(fname, 'wb')
+    with get_file_obj(fname, 'w') as fh:
+        X = np.asarray(X)
+
+        # Handle 1-dimensional arrays
+        if X.ndim == 1:
+            # Common case -- 1d array of numbers
+            if X.dtype.names is None:
+                X = np.atleast_2d(X).T
+                ncol = 1
+
+            # Complex dtype -- each field indicates a separate column
+            else:
+                ncol = len(X.dtype.descr)
         else:
-            fh = file(fname, 'w')
-    elif hasattr(fname, 'seek'):
-        fh = fname
-    else:
-        raise ValueError('fname must be a string or file handle')
+            ncol = X.shape[1]
 
-    X = np.asarray(X)
-
-    # Handle 1-dimensional arrays
-    if X.ndim == 1:
-        # Common case -- 1d array of numbers
-        if X.dtype.names is None:
-            X = np.atleast_2d(X).T
-            ncol = 1
-
-        # Complex dtype -- each field indicates a separate column
-        else:
-            ncol = len(X.dtype.descr)
-    else:
-        ncol = X.shape[1]
-
-    # `fmt` can be a string with multiple insertion points or a list of formats.
-    # E.g. '%10.5f\t%10d' or ('%10.5f', '$10d')
-    if isinstance(fmt, (list, tuple)):
-        if len(fmt) != ncol:
-            raise AttributeError('fmt has wrong shape.  %s' % str(fmt))
-        format = delimiter.join(fmt)
-    elif isinstance(fmt, string_types):
-        if fmt.count('%') == 1:
-            fmt = [fmt, ]*ncol
+        # `fmt` can be a string with multiple insertion points or a list of formats.
+        # E.g. '%10.5f\t%10d' or ('%10.5f', '$10d')
+        if isinstance(fmt, (list, tuple)):
+            if len(fmt) != ncol:
+                raise AttributeError('fmt has wrong shape.  %s' % str(fmt))
             format = delimiter.join(fmt)
-        elif fmt.count('%') != ncol:
-            raise AttributeError('fmt has wrong number of %% formats.  %s'
-                                 % fmt)
-        else:
-            format = fmt
+        elif isinstance(fmt, string_types):
+            if fmt.count('%') == 1:
+                fmt = [fmt, ]*ncol
+                format = delimiter.join(fmt)
+            elif fmt.count('%') != ncol:
+                raise AttributeError('fmt has wrong number of %% formats.  %s'
+                                     % fmt)
+            else:
+                format = fmt
 
-    # handle names
-    if names is None and X.dtype.names:
-        names = X.dtype.names
-    if names is not None:
-        fh.write(delimiter.join(names) + '\n')
+        # handle names
+        if names is None and X.dtype.names:
+            names = X.dtype.names
+        if names is not None:
+            fh.write(delimiter.join(names) + '\n')
 
-    for row in X:
-        fh.write(format % tuple(row) + '\n')
-
-if __name__ == "__main__":
-    import os
-    curdir = os.path.dirname(os.path.abspath(__file__))
-    res1 = genfromdta(curdir+'/../../datasets/macrodata/macrodata.dta')
+        for row in X:
+            fh.write(format % tuple(row) + '\n')
