@@ -50,19 +50,22 @@ License: BSD (3-clause)
 
 from __future__ import print_function
 from statsmodels.compat.python import lrange
+
 import numpy as np
 from scipy import optimize, stats
-from statsmodels.tools.numdiff import approx_fprime, approx_hess
+
+from statsmodels.tools.numdiff import approx_fprime
 from statsmodels.base.model import (Model,
                                     LikelihoodModel, LikelihoodModelResults)
 from statsmodels.regression.linear_model import (OLS, RegressionResults,
                                                  RegressionResultsWrapper)
 import statsmodels.stats.sandwich_covariance as smcov
-from statsmodels.tools.decorators import (resettable_cache, cache_readonly)
-from statsmodels.compat.numpy import np_matrix_rank
+from statsmodels.tools.decorators import cache_readonly
+from statsmodels.tools.tools import _ensure_2d
 
 DEBUG = 0
 
+
 def maxabs(x):
     '''just a shortcut to np.abs(x).max()
     '''
@@ -70,38 +73,36 @@ def maxabs(x):
 
 
 class IV2SLS(LikelihoodModel):
-    '''
-    Class for instrumental variables estimation using Two-Stage Least-Squares
+    """
+    Instrumental variables estimation using Two-Stage Least-Squares (2SLS)
 
 
     Parameters
     ----------
-    endog: array 1d
-       endogenous variable
+    endog: array
+       Endogenous variable, 1-dimensional or 2-dimensional array nobs by 1
     exog : array
-       explanatory variables
-    instruments : array
-       instruments for explanatory variables, needs to contain those exog
-       variables that are not instrumented out
+       Explanatory variables, 1-dimensional or 2-dimensional array nobs by k
+    instrument : array
+       Instruments for explanatory variables. Must contain both exog
+       variables that are not being instrumented and instruments
 
     Notes
     -----
     All variables in exog are instrumented in the calculations. If variables
-    in exog are not supposed to be instrumented out, then these variables
-    need also to be included in the instrument array.
+    in exog are not supposed to be instrumented, then these variables
+    must also to be included in the instrument array.
 
     Degrees of freedom in the calculation of the standard errors uses
     `df_resid = (nobs - k_vars)`.
     (This corresponds to the `small` option in Stata's ivreg2.)
-
-
-    '''
+    """
 
     def __init__(self, endog, exog, instrument=None):
-        self.instrument = instrument
+        self.instrument, self.instrument_names = _ensure_2d(instrument, True)
         super(IV2SLS, self).__init__(endog, exog)
         # where is this supposed to be handled
-        #Note: Greene p.77/78 dof correction is not necessary (because only
+        # Note: Greene p.77/78 dof correction is not necessary (because only
         #       asy results), but most packages do it anyway
         self.df_resid = self.exog.shape[0] - self.exog.shape[1]
         #self.df_model = float(self.rank - self.k_constant)
@@ -112,6 +113,7 @@ class IV2SLS(LikelihoodModel):
         self.wexog = self.exog
 
     def whiten(self, X):
+        """Not implemented"""
         pass
 
     def fit(self):
@@ -152,13 +154,12 @@ class IV2SLS(LikelihoodModel):
                        normalized_cov_params=self.normalized_cov_params)
 
         lfit.exog_hat_params = xhatparams
-        lfit.exog_hat = xhat # TODO: do we want to store this, might be large
-        self._results = lfit  # TODO : remove this
+        lfit.exog_hat = xhat  # TODO: do we want to store this, might be large
         self._results_ols2nd = OLS(y, xhat).fit()
 
         return RegressionResultsWrapper(lfit)
 
-    #copied from GLS, because I subclass currently LikelihoodModel and not GLS
+    # copied from GLS, because I subclass currently LikelihoodModel and not GLS
     def predict(self, params, exog=None):
         """
         Return linear predicted values from a design matrix.
@@ -180,16 +181,8 @@ class IV2SLS(LikelihoodModel):
         """
         if exog is None:
             exog = self.exog
-        return np.dot(exog, params)
-        #JP: this doesn't look correct for GLMAR
-        #SS: it needs its own predict method
-        if self._results is None and params is None:
-            raise ValueError("If the model has not been fit, then you must specify the params argument.")
-        if self._results is not None:
-            return np.dot(exog, self._results.params)
-        else:
-            return np.dot(exog, params)
 
+        return np.dot(exog, params)
 
 
 class IVRegressionResults(RegressionResults):
@@ -212,18 +205,22 @@ class IVRegressionResults(RegressionResults):
 
     @cache_readonly
     def fvalue(self):
-        k_vars = len(self.params)
-        restriction = np.eye(k_vars)
-        idx_noconstant = lrange(k_vars)
-        del idx_noconstant[self.model.data.const_idx]
-        fval = self.f_test(restriction[idx_noconstant]).fvalue # without constant
-        return fval
+        const_idx = self.model.data.const_idx
+        # if constant is implicit or missing, return nan see #2444, #3544
+        if const_idx is None:
+            return np.nan
+        else:
+            k_vars = len(self.params)
+            restriction = np.eye(k_vars)
+            idx_noconstant = lrange(k_vars)
+            del idx_noconstant[const_idx]
+            fval = self.f_test(restriction[idx_noconstant]).fvalue # without constant
+            return fval
 
 
     def spec_hausman(self, dof=None):
         '''Hausman's specification test
 
-
         See Also
         --------
         spec_hausman : generic function for Hausman's specification test
@@ -245,7 +242,7 @@ class IVRegressionResults(RegressionResults):
         #use linalg.lstsq or svd directly
         #cov_diff will very often be in-definite (singular)
         if not dof:
-            dof = np_matrix_rank(cov_diff)
+            dof = np.linalg.matrix_rank(cov_diff)
         cov_diffpinv = np.linalg.pinv(cov_diff)
         H = np.dot(params_diff, np.dot(cov_diffpinv, params_diff))/se2
         pval = stats.chi2.sf(H, dof)
@@ -258,7 +255,7 @@ class IVRegressionResults(RegressionResults):
         """Summarize the Regression Results
 
         Parameters
-        -----------
+        ----------
         yname : string, optional
             Default is `y`
         xname : list of strings, optional
@@ -296,7 +293,7 @@ class IVRegressionResults(RegressionResults):
         condno = np.sqrt(eigvals[-1]/eigvals[0])
 
         # TODO: check what is valid.
-        # box-pierce, breush-pagan, durbin's h are not with endogenous on rhs
+        # box-pierce, breusch-pagan, durbin's h are not with endogenous on rhs
         # use Cumby Huizinga 1992 instead
         self.diagn = dict(jb=jb, jbpv=jbpv, skew=skew, kurtosis=kurtosis,
                           omni=omni, omnipv=omnipv, condno=condno,
@@ -446,9 +443,8 @@ class GMM(Model):
         this is mainly if additional variables need to be stored for the
         calculations of the moment conditions
 
-    Returns
-    -------
-    *Attributes*
+    Attributes
+    ----------
     results : instance of GMMResults
         currently just a storage class for params and cov_params without it's
         own methods
@@ -521,7 +517,7 @@ class GMM(Model):
         # TODO: this is a temporary fix, need
         xnames = self.data.xnames
 
-        if not param_names is None:
+        if param_names is not None:
             if len(params) == len(param_names):
                 self.data.xnames = param_names
             else:
@@ -532,8 +528,32 @@ class GMM(Model):
                 # cut in front for poisson multiplicative
                 self.data.xnames = xnames[-len(params):]
             elif len(params) > len(xnames):
-                # cut at the end
-                self.data.xnames = xnames[:len(params)]
+                # use generic names
+                self.data.xnames = ['p%2d' % i for i in range(len(params))]
+
+    def set_param_names(self, param_names, k_params=None):
+        """set the parameter names in the model
+
+        Parameters
+        ----------
+        param_names : list of strings
+            param_names should have the same length as the number of params
+        k_params : None or int
+            If k_params is None, then the k_params attribute is used, unless
+            it is None.
+            If k_params is not None, then it will also set the k_params
+            attribute.
+        """
+        if k_params is not None:
+            self.k_params = k_params
+        else:
+            k_params = self.k_params
+
+        if k_params == len(param_names):
+            self.data.xnames = param_names
+        else:
+            raise ValueError('param_names has the wrong length')
+
 
     def fit(self, start_params=None, maxiter=10, inv_weights=None,
                   weights_method='cov', wargs=(),
@@ -636,7 +656,7 @@ class GMM(Model):
 
         if optim_args is None:
             optim_args = {}
-        if not 'disp' in optim_args:
+        if 'disp' not in optim_args:
             optim_args['disp'] = 1
 
         if maxiter == 0 or maxiter == 'cue':
@@ -792,6 +812,7 @@ class GMM(Model):
         return optimizer(self.gmmobjective_cu, start, args=(), **optim_args)
 
     def start_weights(self, inv=True):
+        """Create identity matrix for starting weights"""
         return np.eye(self.nmoms)
 
     def gmmobjective(self, params, weights):
@@ -920,9 +941,9 @@ class GMM(Model):
 
         Parameters
         ----------
-        moms : array, (nobs, nmoms)
-            moment conditions for all observations evaluated at a parameter
-            value
+        moms : array
+            moment conditions (nobs x nmoms) for all observations evaluated at
+            a parameter value
         weights_method : string 'cov'
             If method='cov' is cov then the matrix is calculated as simple
             covariance of the moment conditions.
@@ -990,7 +1011,7 @@ class GMM(Model):
         elif weights_method == 'flatkernel':
             #uniform cut-off window
             # This was a trial version, can use HAC with flatkernel
-            if not 'maxlag' in wargs:
+            if 'maxlag' not in wargs:
                 raise ValueError('flatkernel requires maxlag')
 
             maxlag = wargs['maxlag']
@@ -1085,14 +1106,14 @@ class GMM(Model):
         return gradmoms
 
     def score(self, params, weights, epsilon=None, centered=True):
-
+        """Score"""
         deriv = approx_fprime(params, self.gmmobjective, args=(weights,),
                               centered=centered, epsilon=epsilon)
 
         return deriv
 
     def score_cu(self, params, epsilon=None, centered=True):
-
+        """Score cu"""
         deriv = approx_fprime(params, self.gmmobjective_cu, args=(),
                               centered=centered, epsilon=epsilon)
 
@@ -1103,24 +1124,27 @@ class GMM(Model):
 class GMMResults(LikelihoodModelResults):
     '''just a storage class right now'''
 
+    use_t = False
+
     def __init__(self, *args, **kwds):
         self.__dict__.update(kwds)
 
         self.nobs = self.model.nobs
+        self.df_resid = np.inf
 
+        self.cov_params_default = self._cov_params()
 
     @cache_readonly
     def q(self):
+        """Objective function at params"""
         return self.model.gmmobjective(self.params, self.weights)
 
-
     @cache_readonly
     def jval(self):
-        # nobs_moms attached by momcond_mean
+        """nobs_moms attached by momcond_mean"""
         return self.q * self.model.nobs_moms
 
-
-    def cov_params(self, **kwds):
+    def _cov_params(self, **kwds):
         #TODO add options ???)
         # this should use by default whatever options have been specified in
         # fit
@@ -1131,20 +1155,19 @@ class GMMResults(LikelihoodModelResults):
 #             return self._cov_params
 
         # set defaults based on fit arguments
-        if not 'wargs' in kwds:
+        if 'wargs' not in kwds:
             # Note: we don't check the keys in wargs, use either all or nothing
             kwds['wargs'] = self.wargs
-        if not 'weights_method' in kwds:
+        if 'weights_method' not in kwds:
             kwds['weights_method'] = self.options_other['weights_method']
-        if not 'has_optimal_weights' in kwds:
+        if 'has_optimal_weights' not in kwds:
             kwds['has_optimal_weights'] = self.options_other['has_optimal_weights']
 
         gradmoms = self.model.gradient_momcond(self.params)
         moms = self.model.momcond(self.params)
         covparams = self.calc_cov_params(moms, gradmoms, **kwds)
 
-        self._cov_params = covparams
-        return self._cov_params
+        return covparams
 
 
     def calc_cov_params(self, moms, gradmoms, weights=None, use_weights=False,
@@ -1200,7 +1223,7 @@ class GMMResults(LikelihoodModelResults):
         return cov/nobs
 
     @property
-    def bse(self):
+    def bse_(self):
         '''standard error of the parameter estimates
         '''
         return self.get_bse()
@@ -1262,12 +1285,11 @@ class GMMResults(LikelihoodModelResults):
             jdiff = - jdiff
         return jdiff, stats.chi2.sf(jdiff, df), df
 
-
     def summary(self, yname=None, xname=None, title=None, alpha=.05):
         """Summarize the Regression Results
 
         Parameters
-        -----------
+        ----------
         yname : string, optional
             Default is `y`
         xname : list of strings, optional
@@ -1318,13 +1340,13 @@ class GMMResults(LikelihoodModelResults):
         if title is None:
             title = self.model.__class__.__name__ + ' ' + "Results"
 
-        #create summary table instance
+        # create summary table instance
         from statsmodels.iolib.summary import Summary
         smry = Summary()
         smry.add_table_2cols(self, gleft=top_left, gright=top_right,
-                          yname=yname, xname=xname, title=title)
+                             yname=yname, xname=xname, title=title)
         smry.add_table_params(self, yname=yname, xname=xname, alpha=alpha,
-                             use_t=False)
+                              use_t=self.use_t)
 
         return smry
 
@@ -1348,10 +1370,11 @@ class IVGMM(GMM):
     results_class = 'IVGMMResults'
 
     def fitstart(self):
+        """Create array of zeros"""
         return np.zeros(self.exog.shape[1])
 
-
     def start_weights(self, inv=True):
+        """Starting weights"""
         zz = np.dot(self.instrument.T, self.instrument)
         nobs = self.instrument.shape[0]
         if inv:
@@ -1359,21 +1382,21 @@ class IVGMM(GMM):
         else:
             return np.linalg.pinv(zz / nobs)
 
-
     def get_error(self, params):
+        """Get error at params"""
         return self.endog - self.predict(params)
 
-
     def predict(self, params, exog=None):
-
+        """Get prediction at params"""
         if exog is None:
             exog = self.exog
 
         return np.dot(exog, params)
 
     def momcond(self, params):
+        """Error times instrument"""
         instrument = self.instrument
-        return instrument * self.get_error(params)[:,None]
+        return instrument * self.get_error(params)[:, None]
 
 
 class LinearIVGMM(IVGMM):
@@ -1401,7 +1424,7 @@ class LinearIVGMM(IVGMM):
     exog : array_like
         explanatory, right hand side variables, including explanatory variables
         that are endogenous
-    instruments : array_like
+    instrument : array_like
         Instrumental variables, variables that are exogenous to the error
         in the linear model containing both included and excluded exogenous
         variables
@@ -1588,22 +1611,24 @@ class NonlinearIVGMM(IVGMM):
 
 
 class IVGMMResults(GMMResults):
-
-
+    """Results class of IVGMM"""
     # this assumes that we have an additive error model `(y - f(x, params))`
 
     @cache_readonly
     def fittedvalues(self):
+        """Fitted values"""
         return self.model.predict(self.params)
 
 
     @cache_readonly
     def resid(self):
+        """Residuals"""
         return self.model.endog - self.fittedvalues
 
 
     @cache_readonly
     def ssr(self):
+        """Sum of square errors"""
         return (self.resid * self.resid).sum(0)
 
 
@@ -1648,7 +1673,7 @@ def spec_hausman(params_e, params_i, cov_params_e, cov_params_i, dof=None):
     #use linalg.lstsq or svd directly
     #cov_diff will very often be in-definite (singular)
     if not dof:
-        dof = np_matrix_rank(cov_diff)
+        dof = np.linalg.matrix_rank(cov_diff)
     cov_diffpinv = np.linalg.pinv(cov_diff)
     H = np.dot(params_diff, np.dot(cov_diffpinv, params_diff))
     pval = stats.chi2.sf(H, dof)
@@ -1683,7 +1708,7 @@ class DistQuantilesGMM(GMM):
         self.endog = endog
 
         #make this optional for fit
-        if not 'pquant' in kwds:
+        if 'pquant' not in kwds:
             self.pquant = pquant = np.array([0.01, 0.05,0.1,0.4,0.6,0.9,0.95,0.99])
         else:
             self.pquant = pquant = kwds['pquant']
@@ -1792,4 +1817,3 @@ class DistQuantilesGMM(GMM):
 results_class_dict = {'GMMResults': GMMResults,
                       'IVGMMResults': IVGMMResults,
                       'DistQuantilesGMM': GMMResults}  #TODO: should be a default
-
