@@ -5,15 +5,17 @@ Author: Chad Fulton
 License: BSD-3
 """
 from __future__ import division, absolute_import, print_function
-from statsmodels.compat.testing import skip
 
 import warnings
 import os
+
 import numpy as np
+from numpy.testing import assert_equal, assert_allclose
 import pandas as pd
+import pytest
+
 from statsmodels.tools import add_constant
 from statsmodels.tsa.regime_switching import markov_autoregression
-from numpy.testing import assert_equal, assert_allclose
 
 current_path = os.path.dirname(os.path.abspath(__file__))
 
@@ -232,6 +234,7 @@ class MarkovAutoregression(object):
         assert_allclose(res.llf, self.true['llf_fit'], atol=self.atol,
                         rtol=self.rtol)
 
+    @pytest.mark.smoke
     def test_fit_em(self, **kwargs):
         # Test EM fitting (smoke test)
         res_em = self.model._fit_em(**kwargs)
@@ -791,7 +794,7 @@ class TestHamiltonAR1SwitchTVTP(MarkovAutoregression):
         super(TestHamiltonAR1SwitchTVTP, cls).setup_class(
             true, rgnp, k_regimes=2, order=1, exog_tvtp=exog_tvtp)
 
-    @skip
+    @pytest.mark.skip
     def test_fit_em(self):
         pass
 
@@ -829,11 +832,11 @@ class TestFilardo(MarkovAutoregression):
             true, endog, k_regimes=2, order=4, switching_ar=False,
             exog_tvtp=exog_tvtp)
 
-    @skip
+    @pytest.mark.skip
     def test_fit(self, **kwargs):
         pass
 
-    @skip
+    @pytest.mark.skip
     def test_fit_em(self):
         pass
 
@@ -874,11 +877,11 @@ class TestFilardoPandas(MarkovAutoregression):
             true, endog, k_regimes=2, order=4, switching_ar=False,
             exog_tvtp=exog_tvtp)
 
-    @skip
+    @pytest.mark.skip
     def test_fit(self, **kwargs):
         pass
 
-    @skip
+    @pytest.mark.skip
     def test_fit_em(self):
         pass
 
